VERSION = "1.1"


import pygame
from menu import cMenu
import time
import itertools
import sys
import os
import inputwrangler
import json
from settings import *
import pickle

class tone():
	def __init__(self):
		pygame.mixer.init(buffer=512)
		if OKSOUND:
			self.snd_ok = pygame.mixer.Sound(os.path.join("Sounds",OKSOUND))
			self.snd_ok.set_volume(OKVOL)
		if NGSOUND:
			self.snd_ng = pygame.mixer.Sound(os.path.join("Sounds",NGSOUND))
			self.snd_ng.set_volume(NGVOL)
		if SELECTSOUND:
			self.snd_mv = pygame.mixer.Sound(os.path.join("Sounds",SELECTSOUND))
			self.snd_mv.set_volume(SELVOL)
		if BKGSOUND:
			self.snd_bkg = pygame.mixer.Sound(os.path.join("Sounds",BKGSOUND))
			self.snd_bkg.set_volume(BKGVOL)
		if EXITSOUND:
			self.snd_ext = pygame.mixer.Sound(os.path.join("Sounds",EXITSOUND))
			self.snd_ext.set_volume(EXITVOL)
		if not SOUND:
			self.ok = self.nosound
			self.ng = self.nosound
			self.move = self.nosound
		else:
			self.snd_bkg.play(loops=-1,fade_ms=1000)
	def ok(self,wait=False):
		self.snd_ok.play()
		if wait:
			while pygame.mixer.get_busy():
				pass
	def ng(self,wait=False):
		self.snd_ng.play()
		if wait:
			while pygame.mixer.get_busy():
				pass
	def move(self,wait=False):
		self.snd_mv.play()
		if wait:
			while pygame.mixer.get_busy():
				pass
	def exit(self,wait=False):
		self.snd_bkg.fadeout(1000)
		self.snd_ext.play()
		while pygame.mixer.get_busy():
			pass
	def nosound(self):
		pass
	def stop(self):
		self.snd_ok.stop()
		self.snd_ng.stop()
		self.snd_mv.stop()
		self.snd_bkg.fadeout(1000)
	def restart(self):
		if not SOUND:
			self.ok = self.nosound
			self.ng = self.nosound
			self.move = self.nosound
		else:
			self.snd_bkg.play(loops=-1)
	def startapp(self):
		self.snd_ok.play()
		self.snd_bkg.fadeout(1000)
		while pygame.mixer.get_busy():
			pass




#setup and load sounds if enabled
soundcontrol = tone()

pygame.init()
pygame.font.init()
pygame.event.set_blocked(pygame.MOUSEMOTION) # ignore mouse movements to increase speed

# Set the width and height of the screen [width,height]
size = [WIDTH, HEIGHT]
if FULLSCREEN:
	screen = pygame.display.set_mode(size,pygame.DOUBLEBUF|pygame.FULLSCREEN)
else:
	screen = pygame.display.set_mode(size,pygame.DOUBLEBUF)
screen.set_alpha(None) # Possible speed increase associated with this
screen.fill(BGCOLOR)
pygame.display.set_caption("HTPC Interface " + VERSION)
txt = pygame.font.Font(None, 72).render("Menus Are Loading...",1, TEXT_UNSELECTED)
txt2 = txt.get_rect(center=(WIDTH/2,HEIGHT/2))
screen.blit(txt,txt2)
pygame.display.update()
pygame.display.flip()# Show the loading text
screen.fill(BGCOLOR) # Set it up to wipe the loading text as soon as we flip again.

#Loop until the user clicks the close button.
done = False

# Used to manage how fast the screen updates
clock = pygame.time.Clock()

rect_list = [] # This is the list that will hold changes to the screen each frame


#Setup quick-jump alphanumeric menu
a = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
AlphabetSelection = cMenu(WIDTH-50,screen, zip(tuple(a),range(36)))

#Load the center lists
MenuList = []

#If AUTOUPDATE, then run the menu file generator first
if AUTOUPDATE:
	import menugen
	menugen.generate()

#Load all the files from the "Menus" directory, load them via JSON, and add them to the list
for f in os.listdir("Menus"):
	infile = open(os.path.join("Menus",f)).read()
	MenuList.append((f,cMenu(400,screen,json.loads(infile))))

MenuList.append(('Exit', -1)) # Add the exit option
MenuLeft = cMenu(TEXTPAD,screen,MenuList) #Setup the left menu
MenuLeft.active = True #It is the menu we need to start on at launch
selectedMenu = 0

#Setup a list to hold the three current menus
Menus = [MenuLeft,MenuList[0][1],AlphabetSelection]
maxMenus = len(Menus)

rect_list = map(lambda(x):x.update(),Menus) # Load each menu's initial display
rect_list = [item for sublist in rect_list for item in sublist] #flatten the list
pygame.display.flip()
pygame.display.update(rect_list)

#initialize the input manager
wrangler = inputwrangler.inputwrangler()


#Main Loop
while done==False:
	rect_list = [] #reset the list of objects changed this frame

	# EVENT PROCESSING STEP
	for event in pygame.event.get(): # User did something
		wrangler.update(event) # This lets the input manager look at keyboard entries, etc.
		if event.type == pygame.QUIT: # If user clicked close
			done=True # Flag that we are done so we exit this loop
	wrangler.update(None) # This lets the input manager look at joysticks and perform some calculations

	if wrangler.InputActive(): #True if there are any inputs currently enganged - They may or may not be ready to tick
		# Move one menu to the right
		if wrangler.Right():
			if selectedMenu < (len(Menus)-1): #don't move if already at end
				#change active menu and play a sound
				Menus[selectedMenu].active = False 
				selectedMenu += 1
				Menus[selectedMenu].active = True

				soundcontrol.move()
				for m in Menus:
					m.update_buttons = True

		# Move one menu to the left
		elif wrangler.Left():
			if selectedMenu > 0: # Don't go past the first menu
				#change active menu and play sound
				Menus[selectedMenu].active = False
				selectedMenu -= 1
				Menus[selectedMenu].active = True

				soundcontrol.move()

				for m in Menus:
					m.update_buttons = True


		elif wrangler.Up():
			#move the selection up by one.  The menu class handles if this is below 0
			soundcontrol.move()
			Menus[selectedMenu].selection -=1
		elif wrangler.Down():
			# move the selection down by one.  The menu class handles if this is above the max
			soundcontrol.move()
			Menus[selectedMenu].selection +=1
		elif wrangler.Ok():
			if Menus[selectedMenu] == AlphabetSelection:
				#If ok is pushed on the alphabet menu, need to change the middle menu's current selection accordingly. Returns True if found
				result = Menus[1].jumpto(AlphabetSelection.menu_items[AlphabetSelection.selection]['text'])
				#Now jump to the middle menu
				if result:
					soundcontrol.ok()
					selectedMenu = 1
					Menus[2].active = False
					Menus[1].active = True

					#Re-draw the whole screen to get rid of any old fragments
					screen.fill(BGCOLOR)

					for m in Menus:
						m.update_buttons = True

					rect_list = map(lambda(x):x.update(),Menus)
					pygame.display.flip()
					wrangler.reset() # Reset the input manager - avoids some weird bugs that should probably be fixed a different way
				else:
					soundcontrol.ng()

			elif Menus[selectedMenu] == MenuLeft:
				# if ok is pushed on the left menu, need to switch the middle menu to the selected one or exit.
				if MenuLeft.selected()['value'] == -1: # Exit
					soundcontrol.exit()
					done = True
					break
					#sys.exit(0)
				#change the middle menu
				Menus[1] = MenuLeft.selected()['value']
				AlphabetSelection.jumpto('A')
				#jump to the middle menu
				selectedMenu = 1
				Menus[0].active = False
				Menus[1].active = True

				soundcontrol.ok()

				#re-draw the whole screen to get rid of any old fragments
				screen.fill(BGCOLOR)
				for m in Menus:
					m.update_buttons = True
				rect_list = map(lambda(x):x.update(),Menus)
				pygame.display.flip()
				wrangler.reset() # Reset the input manager - avoids some weird bugs that should probably be fixed a different way

			else:
				#OK was pressed on the middle menu.
				pygame.display.quit() #close our (fullscreen) window so we don't interfere with anything
				try:
					soundcontrol.startapp()
					os.system(Menus[selectedMenu].selected()['value']) # Try to launch the selected option
				except:
					soundcontrol.ng()
					print "Could not open " + str(Menus[selectedMenu].selected()['value'])

				soundcontrol.restart()
				#Re-initialize the display and update everything that pointed to the old display
				pygame.display.init()
				pygame.event.set_blocked(pygame.MOUSEMOTION)
				if FULLSCREEN:
					screen = pygame.display.set_mode(size,pygame.DOUBLEBUF|pygame.FULLSCREEN)
				else:
					screen = pygame.display.set_mode(size,pygame.DOUBLEBUF)
				for i in MenuList:
					if i[1] != -1:
						i[1].draw_surface = screen
						i[1].background = screen.copy()
				AlphabetSelection.draw_surface = screen
				AlphabetSelection.background = screen.copy()
				MenuLeft.background = screen.copy()
				MenuLeft.draw_surface = screen
				screen.set_alpha(None)
				screen.fill(BGCOLOR)
				for m in Menus:
					m.update_buttons = True
				wrangler.reset()

		rect_list = map(lambda(x):x.update(),Menus) # Update the screen to check for changes


		rect_list = [item for sublist in rect_list for item in sublist] # flatten list

	pygame.display.update(rect_list) # update the screen with changed items
	clock.tick(MAXFRAMERATE)
	

soundcontrol.stop()
pygame.quit ()
