#! /usr/bin/python
import pygame
import time
from settings import *

class cMenu:
	def __init__(self, x, background, buttonList):
		self.menu_items = [] # List of menu items
		self.font = pygame.font.Font(FONT, TEXTSIZE)	 # Font to use

		self.x = x # Top left corner of the menu
		self.y = 0 # relative to the screen/window

		self.selection = 0 # The currently selected button
		self.selection_prev = 0

		self.background = background.copy() # The unedited background image
		self.draw_surface = background # Surface to draw to
		self.centered = False	# True if the menu is centered
		self.centeredOnScreen = False# True if the menu is centered
		self.active = False
		self.max_width = 0

		self.contained_rect = None

		# Now add any buttons that were sent in

		for button in buttonList:
			self.menu_items.append(self.create_button(button))
		self.update_buttons = True

		self.menu_items[0]['selected'] = True

		self.generate_button_images()

	## ---[ jumpto ]---------------------------------------------------------
	## This sets the selected item.  If byid=False, then it expects a string and 
	## goes to the first button whose name starts with that string.  If byid is
	##False, it needs an integer and goes to that item in the list
	def jumpto(self,value,byid=False):
		found = False
		if byid == True:
			self.menu_items[self.selection]['selected'] = False
			self.selection = value
			self.menu_items[self.selection]['selected'] = True
		else:
			value = str(value)[0].lower()
			self.menu_items[self.selection]['selected'] = False
			c = 0
			for i in self.menu_items:
				if i['text'][0].lower() == value:
					self.selection = c
					i['selected'] = True
					found = True
					break
				c += 1
		self.update(None)

		return found

	def create_button(self, button_info):
		# Set the width and height based on the text
		width, height = self.font.size(button_info[0])
		button_rect = pygame.Rect((0, 0), (width, height))

		set_selected = False  # When the button is created, it is not selected

		new_button = {'text'	 : button_info[0],
				'value'	: button_info[1],
				'selected': set_selected,
				'rect'	 : button_rect,
				'offset'  : (0, 0),
				'as_image' : None,
				's_image' : None,
				'au_image' : None,
				'u_image' : None}
		return new_button

	def generate_button_images(self):
		for button in self.menu_items:
			# creat the selected/unselected active/inactive images based on the text
			r = self.font.render
			width  = button['rect'][2]
			height = button['rect'][3]

			button['s_image'] = r(button['text'], True, TEXT_SELECTED)
			button['u_image'] = r(button['text'], True, TEXT_UNSELECTED)
			button['as_image'] = r(button['text'], True, TEXT_SELECTED_ACTIVE)
			button['au_image'] = r(button['text'], True, TEXT_UNSELECTED_ACTIVE)

	def position_buttons(self):
		width = 0
		height = 0
		max_height = 0
		counter = 0
		x_loc = self.x
		y_loc = self.y

		# largest width of text on screen (+- the last one just in case) used for deciding where to re-draw background on update (to avoid redrawing the whole screen)
		self.max_width = max([i['rect'][2] for i in self.menu_items[max(self.selection - MENU_PAD - 1,0):min(self.selection + MENU_PAD + 1,len(self.menu_items))]])
		# all sections are the same height, so just use the first item.
		max_height = self.menu_items[0]['rect'][3]

		# Position the button in relation to each other for items near the selected item; ignore others in interest of time
		for button in self.menu_items[max(self.selection - MENU_PAD,0):min(self.selection + MENU_PAD,len(self.menu_items))]:
			button['offset'] = (x_loc, y_loc)
			y_loc += max_height + TEXTPAD
			counter += 1

		#draw a rect that goes around all the buttons we just arranged
		self.contained_rect = self.menu_items[max(self.selection-MENU_PAD,0)]['rect'].move(self.menu_items[min(self.selection + MENU_PAD,len(self.menu_items)-1)]['offset'])

		# get the position the selected item should be at (centered)
		shift_y = self.menu_items[self.selection]['offset'][1] - self.draw_surface.get_rect()[3]/2
		shift_x = 0

		# Move the buttons to make them centered
		for button in self.menu_items[max(self.selection - MENU_PAD,0):min(self.selection + MENU_PAD,len(self.menu_items))]:
			button['offset'] = (button['offset'][0] - shift_x, button['offset'][1] - shift_y)

	def selected(self):
		return self.menu_items[self.selection]

	def update(self, e=None):
		redraw_full_menu = False

		#s = self.selection

		if self.selection >= len(self.menu_items):
			self.selection = 0
		elif self.selection < 0:
			self.selection = len(self.menu_items)-1

		if self.selection != self.selection_prev:
			self.menu_items[self.selection_prev]['selected'] = False
			self.menu_items[self.selection]['selected'] = True
			self.update_buttons = True
			self.position_buttons()

		rectangle_list = self.draw_buttons()

		self.selection_prev = self.selection

		return rectangle_list


	def draw_buttons(self, redraw_rect = None):
		rect_list = []
		redraw_rect = self.contained_rect

		# If buttons have been changed (added button(s), deleted button(s),
		# changed colors, etc, etc), then we need to update the button locations
		# and images
		if self.update_buttons:
			self.update_buttons = False

			self.position_buttons()

			# redraw the background surface to the area of the rect before we draw the buttons
			if redraw_rect != None:
				srf = pygame.Surface((self.max_width,HEIGHT))
				srf.fill(BGCOLOR)
				offset = (self.x, self.y)
				drawn_rect = self.draw_surface.blit(srf, offset)
				rect_list.append(drawn_rect)

			# Cycle through the buttons, only draw the ones that need to be redrawn
			#for button in self.menu_items:
			for button in self.menu_items[max(self.selection - MENU_PAD,0):min(self.selection + MENU_PAD,len(self.menu_items))]:
				if self.active:
					if button['selected']:
						image = button['as_image']
					else:
						image = button['au_image']
				else:
					if button['selected']:
						image = button['s_image']
					else:
						image = button['u_image']

				drawn_rect = self.draw_surface.blit(image, button['offset'], button['rect'])
				rect_list.append(drawn_rect)

		return rect_list

